#ifndef SH3TIMERS_H
#define SH3TIMERS_H

    /*  Dclarations de fonctions ASM contenues dans la librairie "SH3Timers.lib"
    pour ClassPad300. crites par Alexandre Rion (gaumerie@hotmail.com)     */

#ifdef WIN32
#error "SH3Timers only work on ClassPad300"
#endif

    //  defines for parameters "unsigned char uc_Timer"
#define SH3TIMER_0  0x00
#define SH3TIMER_1  0x01
#define SH3TIMER_2  0x02

    //  defines for parameter "unsigned char uc_ClkFrq" of the SH3Timer_SetClockFreq function
#define SH3TIMER_CLKFRQ_3MHZ    0x00    //  3682419 Hz
#define SH3TIMER_CLKFRQ_920KHZ  0x01    //  920385 Hz
#define SH3TIMER_CLKFRQ_230KHZ  0x02    //  230147 Hz
#define SH3TIMER_CLKFRQ_57KHZ   0x03    //  57538 Hz

    //  defines with clock frequency's real values
#define CLKFRQ_3MHZ     3682419
#define CLKFRQ_920KHZ   920385
#define CLKFRQ_230KHZ   230147
#define CLKFRQ_57KHZ    57538

    /* READ ME */
/* What is a SH3Timer:
        -   A SH3Timer is a processor's 32 bits register that count down with some frequency (clock frequency).
    The processor has 3 such registers.
    When the down counter underflows, the processor set a flag and set a new value in the count down register (timer frequency).
*/
/* How to use SH3Timers :
    -   Start a timer:
        1  Set the clock frequency with the SH3Timer_SetClockFreq function
        2  Set the timer frequency with the SH3Timer_SetTimerFreq function
        3  Set the initial value of the timer with the SH3Timer_SetCount
        4  Start the timer with the SH3Timer_Start or SH3Timer_StartAll functions
    -   Check a timer:
        5  Check if the the timer has underflowed and, in this case, clear the corresponding flag with the SH3Timer_HasUnderflowed function
    -   Stop a timer:
        6  Stop the timer with the SH3Timer_Stop or SH3Timer_StopAll functions
*/
/*  Notes:
    -   SH3Timers can have other clock frequency than the ones specified above , but some shouldn't be used (and can't be set with the SH3Timer_SetClockFreq function). So as you haven't set the clock frequency yet, the SH3Timer_GetClockFreq function can return a value that is not specified above. 
    -   The functions SH3Timer_Start, SH3Timer_Stop, SH3Timer_SetTimerFreq, SH3Timer_SetClockFreq and SH3Timer_SetCount don't do anything when the parameter "unsigned char uc_Timer" is not valid.
    -   The functions SH3Timer_GetTimerFreq, SH3Timer_GetClockFreq, SH3Timer_GetCount and SH3Timer_HasUnderflowed return an undefined value when the parameter "unsigned char uc_Timer" is not valid.
    -   The function SH3Timer_SetClockFreq keep only the low 3 bits of the parameter "unsigned char uc_ClkFrq"
    
    /!\ SH3Timers' count registers can be read when it runs (SH3Timer_GetCount function) but can't be written (SH3Timer_SetCount). You have to stop the timer before to write a value.
    /!\ SH3Timers keep on counting down when the add-in is closed, so don't forget to stop all the timers before to quit (SH3Timer_StopAll function)
    
*/
#ifdef __cplusplus
extern "C"{
#endif

        //  start the corresponding timer
    void SH3Timer_Start(unsigned char uc_Timer);
        //  start the 3 timers
    void SH3Timer_StartAll();
        //  stop the corresponding timer
    void SH3Timer_Stop(unsigned char uc_Timer);
        //  stop the 3 timers
    void SH3Timer_StopAll();
        //  return "true" if the corresponding timer is running
    bool SH3Timer_GetState(unsigned char uc_Timer);

        //  set the timer count clock (clock frequency)
    void SH3Timer_SetClockFreq(unsigned char uc_Timer, unsigned char uc_ClkFrq);
        //  get the timer count clock (clock frequency)
    unsigned char SH3Timer_GetClockFreq(unsigned char uc_Timer);
        //  set the value for setting in the corresponding timer when a count-down result in an under flow (timer frequency)
    void SH3Timer_SetTimerFreq(unsigned char uc_Timer, unsigned int timFrq);
        //  get the value for setting in the corresponding timer when a count-down result in an under flow (timer frequency)
    unsigned int SH3Timer_GetTimerFreq(unsigned char uc_Timer);
        
        //  set the current value of the timer
    void SH3Timer_SetCount(unsigned char uc_Timer, unsigned int count);
        //  get the current value of the timer
    unsigned int SH3Timer_GetCount(unsigned char uc_Timer);

        //  return "true" if the corresponding timer has underflowed and, in this case, clear the corresponding flag
    bool SH3Timer_HasUnderflowed(unsigned char uc_Timer);

#ifdef __cplusplus
}
#endif

#endif
